@@ -7,7 +7,7 @@ from .context import Context
 import grpc
 import threading
 import time
-import urllib3
+import requests
 import sseclient
 import base64
 import prefab_pb2 as Prefab
@@ -62,6 +62,12 @@ class ConfigClient:
 
     def get(self, key, default="NO_DEFAULT_PROVIDED", context=Context.get_current()):
         value = self.__get(key, None, {}, context=context)
+
+        if isinstance(context, Context):
+            self.base_client.context_shape_aggregator.push(context)
+        elif not isinstance(context, str):
+            self.base_client.context_shape_aggregator.push(Context(context))
+
         if value is not None:
             return ConfigValueUnwrapper.unwrap(value, key, context)
         else:
@@ -69,17 +75,18 @@ class ConfigClient:
 
     def __get(self, key, lookup_key, properties, context=Context.get_current()):
         try:
-            with self.init_lock.read_locked():
-                return self.config_resolver.get(key, context=context)
+            self.init_lock.acquire_read(self.options.connection_timeout_seconds)
         except Exception:
             if self.options.on_connection_failure == "RAISE":
                 raise InitializationTimeoutException(
                     self.options.connection_timeout_seconds, key
                 )
-            self.base_client.logger().warn(
-                f"Couldn't initialize in {self.options.connection_timeout_seconds}. Key {key}. Returning what we have."
+            self.base_client.logger.log_internal(
+                "warn",
+                f"Couldn't initialize in {self.options.connection_timeout_seconds}. Key {key}. Returning what we have.",
             )
             self.init_lock.release_write()
+        finally:
             return self.config_resolver.get(key, context=context)
 
     def handle_default(self, key, default):
@@ -109,12 +116,10 @@ class ConfigClient:
         auth = "%s:%s" % ("authuser", self.options.api_key)
         token = base64.b64encode(auth.encode("utf-8")).decode("ascii")
         headers = {
-            "x-prefab-start-at-id": self.config_loader.highwater_mark,
+            "x-prefab-start-at-id": f"{self.config_loader.highwater_mark}",
             "Authorization": "Basic %s" % token,
         }
-        response = urllib3.PoolManager().request(
-            "GET", url, headers=headers, preload_content=False
-        )
+        response = self.base_client.session.get(url, headers=headers, stream=True)
 
         client = sseclient.SSEClient(response)
 
@@ -139,15 +144,15 @@ class ConfigClient:
         auth = "%s:%s" % ("authuser", self.options.api_key)
         token = base64.b64encode(auth.encode("utf-8")).decode("ascii")
         headers = {"Authorization": "Basic %s" % token}
-        response = urllib3.PoolManager().request("GET", url, headers=headers)
-        if response.status == 200:
-            configs = Prefab.Configs.FromString(response.data)
+        response = requests.get(url, headers=headers)
+        if response.ok:
+            configs = Prefab.Configs.FromString(response.content)
             self.load_configs(configs, "remote_api_cdn")
             return True
         else:
             self.base_client.logger.log_internal(
                 "info",
-                f"Checkpoint remote_cdn_api failed to load. Response {response.status}",
+                f"Checkpoint remote_cdn_api failed to load. Response {response.status_code}",
             )
             return False
 
